//
//  NightShiftSwitch.swift
//  OnlySwitch
//
//  Created by Jacklandrin on 2021/12/5.
//

import AppKit

class NightShiftSwitch:SwitchProvider {
    weak var delegate: SwitchDelegate?
    var type: SwitchType = .nightShift
    
    func isVisible() -> Bool {
        return NightShiftTool.supportsNightShift
    }
    
    static let shared = NightShiftSwitch()
    func currentInfo() -> String {
        return ""
    }
    
    func currentStatus() -> Bool {
        return NightShiftTool.isNightShiftEnabled
    }
    
    func operateSwitch(isOn: Bool) async throws {
        if isOn {
            NightShiftTool.enable()
            NightShiftTool.strength = 1
        } else {
            NightShiftTool.strength = 0
            NightShiftTool.disable()
        }
    }
}

class NightShiftTool {
    private static let client = CBBlueLightClient()

    private static var blueLightStatus: Status {
        var status: Status = Status()
        client.getBlueLightStatus(&status)
        return status
    }

    static var strength: Float {
        get {
            var strength: Float = 0
            client.getStrength(&strength)
            return strength
        }
        set {
            client.setStrength(newValue, commit: true)
        }
    }

    static func previewStrength(_ value: Float) {
        // Check if user manually disabled Night Shift
        if !isNightShiftEnabled { isNightShiftEnabled = true }
        client.setStrength(value, commit: false)
    }

    static var isNightShiftEnabled: Bool {
        get { return blueLightStatus.enabled.boolValue }
        set { client.setEnabled(newValue) }
    }

    public static func enable() {
        isNightShiftEnabled = true
    }

    public static func disable() {
        isNightShiftEnabled = false
    }

    static var supportsNightShift: Bool { return CBBlueLightClient.supportsBlueLightReduction() }
}
